#include "TCA9534.h"

TCA9534::TCA9534(MinionI2cBus& i2cBus, DeviceInfoStruct& deviceInfo) : MinionI2cBaseDevice(i2cBus, deviceInfo)
{
	// Set Register Default Values:
	_polarityInversion = 0b00000000;	// do not invert pin polarity of the pins configured as inputs
	_configuration = 0b11111111;		// all inputs (with hight-impedance output driver)
	_outputPort = 0b11111111;			// all outputs high
}

void TCA9534::Setup()
{
	_setupRun = true;
	// Configure TCA9534 Registers
	bool success = false;
	success = WriteRegister(PolarityInversionRegister, _polarityInversion);
	success &= _writeConfigurationRegister();
	success &= _writeOutputPortRegister();
}

bool TCA9534::_writeConfigurationRegister()
{
	return WriteRegister(ConfigurationRegister, _configuration);
}

bool TCA9534::_writeOutputPortRegister()
{
	return WriteRegister(OutputPortRegister, _outputPort);
}

bool TCA9534::ReadInputChannel(int channelNr) {
	ReadRegister(InputPortRegister);
	int mask = 1 << channelNr;
	int masked_n = _inputPort & mask;
	int theBit = masked_n >> channelNr;
	return (bool)theBit;
}

bool TCA9534::ChangeOutputChannelValue(int channelNr, bool value) {
	// TODO: Don't change output cofiguration here!
	//ChangeChannelConfiguration(channelNr, OUTPUT);
	bool success = true;
	if (value) _outputPort |= 1U << channelNr;		// set output channel high
	else _outputPort &= ~(1U << channelNr);			// clear output channel
	if (_setupRun) success = _writeOutputPortRegister();
	return success;
}

bool TCA9534::ChangeChannelConfiguration(int channelNr, uint8_t mode) {
	bool success = true;
	if (mode == INPUT) {
		_configuration |= 1U << channelNr;		// set configuration as input (high)
	}
	else {
		_configuration &= ~(1U << channelNr);	// set configuration as output (low)
	}
	if (_setupRun) success = _writeConfigurationRegister();
	return success;
}

void TCA9534::ChangeChannelInputInverstion(int channelNr, bool inverted) {
	if (inverted) {
		_polarityInversion |= 1U << channelNr;		// invert input polarity (high)
	}
	else {
		_polarityInversion &= ~(1U << channelNr);	// don't invert input polarity (low)
	}
}

bool TCA9534::WriteRegister(Tca9534Register tca9534Register, uint8_t registerContent) {
	uint8_t sendContent[2] = { tca9534Register, registerContent };
	return Transmit(ModeWrite, sendContent, 0, sizeof(sendContent));
}

uint8_t TCA9534::ReadRegister(Tca9534Register tca9534Register) {
	uint8_t tmpBuffer;
	bool success = true;
	success &= Transmit(ModeWrite, (uint8_t*)&tca9534Register, 0, 1);
	success &= Transmit(ModeRead, &tmpBuffer, 1, 0);

	switch (tca9534Register) {
	case 0:
		_inputPort = tmpBuffer;
		break;
	case 1:
		_outputPort = tmpBuffer;
		break;
	case 2:
		_polarityInversion = tmpBuffer;
		break;
	case 3:
		_configuration = tmpBuffer;
		break;
	default:
		break;
	}
	return tmpBuffer;
}

