#include "TCA9534Gpio.h"


TCA9534Gpio::TCA9534Gpio(TCA9534& tca9534, GpioExpanderPort portNr, uint8_t mode) : MinionGpio(portNr, TCA9534GpioType, mode), _tca9534(tca9534) {
	_inputInverted = false;
	_tca9534.ChangeChannelConfiguration(_gpioNr, _mode);
}

TCA9534Gpio::TCA9534Gpio(TCA9534& tca9534, GpioExpanderPort portNr, uint8_t mode, uint8_t initialValue) : MinionGpio(portNr, TCA9534GpioType, mode), _tca9534(tca9534) {
	_inputInverted = false;
	_tca9534.ChangeChannelConfiguration(_gpioNr, mode);
	_value = initialValue;
	_valueInitialized = true;
	_tca9534.ChangeOutputChannelValue(_gpioNr, (bool)_value);
}

int TCA9534Gpio::Read() {
	uint8_t tmpValue;
	if (_mode == INPUT) {		// when configured as Output: return value = last set value and not currently read value -> otherwise toggle is not working!!!
		tmpValue = _tca9534.ReadInputChannel(_gpioNr);
		_value = tmpValue;
		_valueInitialized = true;
	}
	else
	{
		if (!_valueInitialized)
		{
			tmpValue = _tca9534.ReadInputChannel(_gpioNr);
			_valueInitialized = true;
		}
	}
	return _value;
}

bool TCA9534Gpio::Write(uint8_t value) {
	_value = value;
	bool success = _tca9534.ChangeChannelConfiguration(_gpioNr, _mode);
	success &= _tca9534.ChangeOutputChannelValue(_gpioNr, (bool)_value);
	return success;
}

bool TCA9534Gpio::Toggle() {
	_tca9534.ChangeChannelConfiguration(_gpioNr, _mode);
	bool success;
	if (!_valueInitialized) _value = Read();
	if (_value == HIGH) success = Write(LOW);
	else success = Write(HIGH);
	return success;
}

bool TCA9534Gpio::SetMode(uint8_t mode) {
	_mode = mode;
	return _tca9534.ChangeChannelConfiguration(_gpioNr, _mode);
}
