
#include <Arduino.h>
#include <Wire.h>
#include <MinionI2cBus.h>
#include <MinionI2cBaseDevice.h>
#include <TCA9534Gpio.h>
#include <TCA9534.h>
#include <DirectGpio.h>

MinionI2cBusDefinitionStruct _i2cBus{
	STANDARD_MODE_SPEED,            // Max Bus Speed
	0,                              // Number of Multiplexers
	{}                              // Sub Buses
};

TwoWire _twoWire(0);
MinionI2cBus _i2c(_twoWire, _i2cBus, SDA, SCL);

DirectGpio TasterIntern(INPUT_ONLY_GPIO37, INPUT);		// TasterIntern ist beim MinionS001R1 auf GPIO 37 angeschlossen
DirectGpio WinkLed(GPIO2, OUTPUT);						// Die Wink-LED ist bei den Minions auf GPIO 2 angeschlossen

DeviceInfoStruct tca9534Info = { EMPTY_DEVICE_PATH, 0x20, FAST_MODE_SPEED };	
TCA9534 GpioExpander(_i2c, tca9534Info);				// Der GpioExpander (TCA9534) ist beim MINIONS001R1 auf I2c Addresse 0x20
TCA9534Gpio Testpoint(GpioExpander, P6, OUTPUT, HIGH);	// Am Channel 6 des GPIO Expanders ist ein Testpunkt

void setup() {
	Serial.begin(115200);
	_i2c.Setup();
	GpioExpander.Setup();
	WinkLed.Write(LOW);						// Initialisiere Wink-LED auf low
	Testpoint.Write(HIGH);					// Initialisiere Testpunkt auf high
}

// ACHTUNG!!!
// Beim MINIONS001R1 fehlt beim TasterIntern der Pullup, folglich ist der Wert immer Low und das Programm toggelt immer langsam

void loop() {
	if (TasterIntern.Read() == LOW) {
		// Taster gedrückt -> langsam wechseln
		delay(1000);
	}
	else {
		// Taster nicht gedrückt -> schnell wechseln
		delay(100);
	}
	WinkLed.Toggle();
	Testpoint.Toggle();
	Serial.print("WinkLed: "); Serial.println(WinkLed.Read());
	Serial.print("Testpoint: "); Serial.println(Testpoint.Read());
}