/*
 Name:		MinionI2cBus.cpp
 Created:	25.04.2019 17:35:56
 Author:	aoetiker
 Editor:	http://www.visualmicro.com
*/

#include "MinionI2cBus.h"

MinionI2cBus::MinionI2cBus(TwoWire& i2c) : MinionI2cBus(i2c, SIMPLE_BUS, SDA, SCL)
{
}


MinionI2cBus::MinionI2cBus(TwoWire& i2c, MinionI2cBusDefinitionStruct busDefinition) : MinionI2cBus(i2c, busDefinition, SDA, SCL)
{
}


MinionI2cBus::MinionI2cBus(TwoWire& i2c, MinionI2cBusDefinitionStruct busDefinition, int sdaPin, int sclPin)
    : _i2c(i2c), _busDefinition(busDefinition), _sdaPin(sdaPin), _sclPin(sclPin)
{
    _i2c.begin(_sdaPin, _sclPin, _busDefinition.MaxBusSpeed);
    // TODO: Recursively set current selected channel of each Multiplexer to -1
	_minionI2cBusSetupRun = false;
}

int MinionI2cBus::Scan(uint8_t* buffer)
{
    byte error;
    int nDevices = 0;

    for (int i = 1; i < 127; i++)
    {
        _i2c.beginTransmission(i);
        error = _i2c.endTransmission();
        if (error == 0)
        {
            buffer[nDevices] = i;
            nDevices++;
        }
        else if (error == 2)
        {
            //Serial.print("received NACK on transmit of address 0x");
            //if (i < 0x10)
            //{
            //    Serial.print("0");
            //}
            //Serial.println(i, HEX);
        }
        else if (error == 4)
        {
            //Serial.print("other error on transmit of address 0x");
            //if (i < 0x10)
            //{
            //    Serial.print("0");
            //}
            //Serial.println(i, HEX);
        }
    }
    return nDevices;
}



bool MinionI2cBus::SelectChannel(MultiplexerStruct* multiplexer, int channel, uint32_t maxBusSpeed)
{
    //Serial.println("*** SelectChannel BEGIN ***");
    //Serial.print("channel: "); Serial.println(channel);
    //Serial.print("maxBusSpeed: "); Serial.println(maxBusSpeed);

    if (channel > multiplexer->NumberOfChannels - 1)
    {
        return false;
    }
    
    switch(multiplexer->Type)
    {
        case TCA9546:
            {
                //Serial.println("TCA9546");
                uint32_t speed = (maxBusSpeed < FAST_MODE_SPEED) ? maxBusSpeed : FAST_MODE_SPEED;
                _i2c.setClock(speed);
                //Serial.print("setClock to: "); Serial.println(speed);
                _i2c.beginTransmission(multiplexer->I2cAddress);
                //Serial.print("beginTransmission to: "); Serial.println(multiplexer->I2cAddress);
                // Channel 0 = 0x01, Channel 1 = 0x02, Channel 2 = 0x04, Channel 3 = 0x08, All Channels disabled: 0x00
                uint8_t data = (channel >= 0) ? (0x01 << channel) : 0x00;
                _i2c.write(data);
                //Serial.print("write: "); Serial.println(data);
                _i2c.endTransmission();
            }
            break;

        default:
            break;
    }
    multiplexer->CurrentSelectedChannel = channel;

    //Serial.println("*** SelectChannel END ***");
    return true;
}



bool MinionI2cBus::SwitchToDevice(DevicePathStruct devicePath, uint32_t& maxBusSpeed)

{
    //Serial.println("*** SwitchToDevice BEGIN ***");
    MinionI2cBusDefinitionStruct* currentBusDefinition = &_busDefinition;
    //uint32_t maxBusSpeed = currentBusDefinition->MaxBusSpeed;
    //Serial.print("PathDepth: "); Serial.println(devicePath.PathDepth);
    for (int i = 0; i < devicePath.PathDepth; i++)
    {
        
        uint8_t arrayIndex = devicePath.Path[i].MultiplexerIndex;
        //Serial.print("arrayIndex: "); Serial.println(devicePath.Path[i].MultiplexerIndex);
        int channel = devicePath.Path[i].MultiplexerChannel;
        //Serial.print("channel: "); Serial.println(devicePath.Path[i].MultiplexerChannel);

        if (currentBusDefinition->Multiplexers[arrayIndex]->CurrentSelectedChannel != channel)
        {
            //Serial.print("setClock to: "); Serial.println(maxBusSpeed);

            //Serial.println("current selected != channel");
            if (!SelectChannel(currentBusDefinition->Multiplexers[arrayIndex], channel, maxBusSpeed))
            {
                //Serial.println("return false");
                return false;
            }
        }
        currentBusDefinition = currentBusDefinition->Multiplexers[arrayIndex]->SubBuses[channel];
        maxBusSpeed = (maxBusSpeed < currentBusDefinition->MaxBusSpeed) ? maxBusSpeed : currentBusDefinition->MaxBusSpeed;
    }

    //Serial.println("*** SwitchToDevice END ***");
    return true;

}

void MinionI2cBus::Setup()
{
	_minionI2cBusSetupRun = true;
}

bool MinionI2cBus::GenericMessage(DeviceInfoStruct* deviceInfo, uint8_t mode, uint8_t* buffer, uint8_t readSize, uint8_t writeSize)
{
    //Serial.println("*** GenericMessage BEGIN***");
    bool withoutStop = false;
    
    uint32_t maxBusSpeed = _busDefinition.MaxBusSpeed;
    if (!SwitchToDevice(deviceInfo->DevicePath, maxBusSpeed))
    {
        return false;
    }

    uint32_t speed = (maxBusSpeed < deviceInfo->Speed) ? maxBusSpeed : deviceInfo->Speed;
    _i2c.setClock(speed);


    if (mode == ModeWriteWithoutStop || mode == ModeReadWithoutStop)
    {
        withoutStop = true;
    }

	if (!_minionI2cBusSetupRun)
	{
		Serial.end();
		delay(1000);
		Serial.begin(115200);
		while (true) 
		{ 
			Serial.println("i2c setup not yet run"); 
		}
	}

    // Write
    if (mode == ModeWrite || mode == ModeWriteWithoutStop)
    {
        _i2c.beginTransmission(deviceInfo->I2cAddress);
        for (int i = 0; i < writeSize; i++)
        {
            _i2c.write(buffer[i]);
        }


        if (_i2c.endTransmission(!withoutStop))
        {
            return false;
        }

    }


    // Read
    if (mode == ModeRead || mode == ModeReadWithoutStop)
    {
        memset(buffer, 0x00, readSize);
        _i2c.requestFrom((uint16_t)deviceInfo->I2cAddress, readSize, !withoutStop);
        for (uint8_t i = 0; i < readSize; i++)
        {
            if (_i2c.available())
            {
                buffer[i] = _i2c.read();
            }

        }
    }
    //Serial.println("*** GenericMessage END ***");
    return true;
}

bool MinionI2cBus::CheckDeviceConnection(DeviceInfoStruct* deviceInfo)
{
    //Serial.println("*** CheckDeviceConnection BEGIN***");

    uint32_t maxBusSpeed = _busDefinition.MaxBusSpeed;
    if (!SwitchToDevice(deviceInfo->DevicePath, maxBusSpeed))
    {
        //Serial.println("SwitchToDevice failed");
        return false;
    }
    //Serial.println("SwitchToDevice successful");


    uint32_t speed = (maxBusSpeed < deviceInfo->Speed) ? maxBusSpeed : deviceInfo->Speed;
    //Serial.println(maxBusSpeed);
    //Serial.println(deviceInfo->Speed);
    //Serial.println(speed);
    _i2c.setClock(speed);
    //Serial.print("setClock to: "); Serial.println(speed);

    _i2c.beginTransmission(deviceInfo->I2cAddress);
    //Serial.print("beginTransmission to: "); Serial.println(deviceInfo->I2cAddress);


    uint8_t error;
    error = _i2c.endTransmission();
    //Serial.println(error);
    if (error > 0) return false;
        
    //Serial.println("*** CheckDeviceConnection END***");
    return true;
}

