/*
 Name:		MinionI2cBus.h
 Created:	25.04.2019 17:32:25
 Author:	aoetiker
 Editor:	http://www.visualmicro.com
*/

#ifndef _MinionI2cBus_h
#define _MinionI2cBus_h


#if defined(ARDUINO) && ARDUINO >= 100
	#include "arduino.h"
#else
	#include "WProgram.h"
#endif

#include <Wire.h>

#define SDA 5
#define SCL 4

#define STANDARD_MODE_SPEED 100000
#define FAST_MODE_SPEED 400000
#define FAST_MODE_PLUS_SPEED 1000000
#define HIGH_SPEED_MODE_SPEED 3400000

#define MAX_PATH_DEPTH 4
#define MAX_NUMBER_OF_MULTIPLEXERS 8
#define MAX_NUMBER_OF_CHANNELS 8

#define EMPTY_DEVICE_PATH { 0, { {} } }
#define SIMPLE_BUS { FAST_MODE_PLUS_SPEED , 0, {} }
             

enum I2cMode
{
	ModeRead = 0,
	ModeWrite = 1,
	ModeReadWithoutStop = 2,
	ModeWriteWithoutStop = 3,
};


enum I2cMultiplexerType
{
    TCA9546 = 0
};



struct MultiplexerStruct; // forward declaration

struct MinionI2cBusDefinitionStruct
{
    uint32_t MaxBusSpeed;
    uint8_t NumberOfMultiplexers;
    MultiplexerStruct* Multiplexers[MAX_NUMBER_OF_MULTIPLEXERS];
};


struct MultiplexerStruct
{
    uint8_t I2cAddress;
    uint8_t NumberOfChannels;
    I2cMultiplexerType Type;
    int CurrentSelectedChannel;
    MinionI2cBusDefinitionStruct* SubBuses[MAX_NUMBER_OF_CHANNELS];
};


struct MultiplexerConfigurationStruct
{
    uint8_t MultiplexerIndex;
    int MultiplexerChannel;
};


struct DevicePathStruct
{
    uint8_t PathDepth;
    MultiplexerConfigurationStruct Path[MAX_PATH_DEPTH];
};


struct DeviceInfoStruct
{
    DevicePathStruct DevicePath;
    uint8_t I2cAddress;
    uint32_t Speed;
};



class MinionI2cBus
{

public:
	MinionI2cBus(TwoWire& i2c);
	MinionI2cBus(TwoWire& i2c, MinionI2cBusDefinitionStruct busDefinition);
    MinionI2cBus(TwoWire& i2c, MinionI2cBusDefinitionStruct busDefinition, int sdaPin, int sclPin);

    //ScanResult* ScanBus();
	void Setup();
    bool GenericMessage(DeviceInfoStruct* deviceInfo, uint8_t mode, uint8_t* buffer, uint8_t readSize, uint8_t writeSize);
    bool CheckDeviceConnection(DeviceInfoStruct* deviceInfo);
    bool SwitchToDevice(DevicePathStruct devicePath, uint32_t& maxBusSpeed); // Public, because of 3rd Party I2c Devices
    int Scan(uint8_t* buffer); // Scans the current set bus

private:

    TwoWire& _i2c;
    MinionI2cBusDefinitionStruct _busDefinition;
    int _sdaPin;
    int _sclPin;
	bool _minionI2cBusSetupRun;
    bool SelectChannel(MultiplexerStruct* multiplexer, int channel, uint32_t maxBusSpeed);
};


#endif

