/*
 Name:		Connected.ino
 Created:	30.04.2019 18:36:12
 Author:	aoetiker
*/

/* 
This example checks the connection status of the defined Didroms. The example works with any Minion Pcb.
The Multiplexer Bus was defined for the MINIONS002R0 Pcb with two external Acoustic Buses.
The Bus mode can be changed thru the three defines below.
*/

#include <Wire.h>
#include <MinionI2cBaseDevice.h>

MinionI2cBusDefinitionStruct _simpleBus{
    STANDARD_MODE_SPEED,            // Max Bus Speed
    0,                              // Number of Multiplexers
    {}                              // Sub Buses
};

MinionI2cBusDefinitionStruct _simpleBus_Acoustic10kHz{
    10000,                          // Max Bus Speed
    0,                              // Number of Multiplexers
    {}                              // Sub Buses
};

MultiplexerStruct _externalI2CMultiplexer
{
    0x70,                           // Address
    4,                              // Number of Channels
    TCA9546,                        // Type
    -1,                             // Current selected Channel

    {                               // Sub Buses:
        &_simpleBus_Acoustic10kHz,  // Extern Channel 0, Acoustic 1, JP1055
        &_simpleBus_Acoustic10kHz,  // Extern Channel 1, Acoustic 2, JP1056
        &_simpleBus,                // Extern Channel 2, I2c Ext 2,  JP1054
        &_simpleBus                 // Extern Channel 3, I2c Ext 3,  JP1052
    }
};

MinionI2cBusDefinitionStruct _i2cBusDefinition
{
    STANDARD_MODE_SPEED,            // Max Bus Speed
    1,                              // Number of Multiplexers
    {
        &_externalI2CMultiplexer
    }
};

// ---------------------------------------------------------------
// Defines to change the Bus Mode. Only one define can be active:
// ---------------------------------------------------------------
//#define SimpleStandardBus         // Standard TwoWire Bus without Multiplexers and standard pin settings (Only a EMPTY_DEVICE_PATH is allowed)
//#define MultiplexerBusStandard    // Standard TwoWire Bus with _busdefinition and standard pin settings
#define MultiplexerBusConfigurable  // Specific TwoWire Bus with _busdefinition and the pins configurable. A TwoWire Bus must be created in advance.


TwoWire _twoWire(0);
#if defined SimpleStandardBus
MinionI2cBus _i2c (_twoWire);
#endif

#if defined MultiplexerBusStandard
MinionI2cBus _i2c(_twoWire, _i2cBusDefinition);
#endif

#if defined MultiplexerBusConfigurable

MinionI2cBus _i2c(_twoWire, _i2cBusDefinition, SDA, SCL);
#endif
// ---------------------------------------------------------------



MinionI2cBaseDevice DidRom(_i2c, { EMPTY_DEVICE_PATH, 0x50, FAST_MODE_SPEED });

#if (defined MultiplexerBusConfigurable) | (defined MultiplexerBusStandard)
MinionI2cBaseDevice DidRomAcoustic1(_i2c, { { 1, { { 0, 0 } } }, 0x51, FAST_MODE_SPEED });
MinionI2cBaseDevice DidRomAcoustic2(_i2c, { { 1, { { 0, 1 } } }, 0x51, FAST_MODE_SPEED });
#endif

void setup()
{
    Serial.begin(115200);
    while (!Serial);
}

void loop() {

    bool connected;

    connected = DidRom.Connected();
    Serial.print("DidRom Connection Status: ");
    connected ? Serial.println("true") : Serial.println("false");


    #if (defined MultiplexerBusConfigurable) | (defined MultiplexerBusStandard)
    connected = DidRomAcoustic1.Connected();
    Serial.print("DidRomAcoustic1 Connection Status: ");
    connected ? Serial.println("true") : Serial.println("false");

    connected = DidRomAcoustic2.Connected();
    Serial.print("DidRomAcoustic2 Connection Status: ");
    connected ? Serial.println("true") : Serial.println("false");
    #endif

    Serial.println("");

    delay(2000);
}
